/*!\file window.c
 *
 * \brief sdlGLexemple01 : utilisation d'OpenGL avec la sdl ;
 * exemple minimaliste.
 *
 * \author : Farès BELHADJ (Lire COPYING) amsi@ai.univ-paris8.fr 
 *
 * \date : September 26, 2013
 */

#include <SDL.h>
#include <GL/gl.h>
#include <GL/glu.h>

static SDL_Surface * sdlInit(int w, int h);
static void          sdlInitGL(SDL_Surface * s);
static void          sdlLoop(SDL_Surface * s);
static void          sdlManageEvents(SDL_Surface * s);
static void          sdlDraw(SDL_Surface * s);

/*!\brief initialise la surface d'affichage puis appelle la boucle
 * principale de gestioon d'évènement/dessin */
int main(int argc, char ** argv) {
  SDL_Surface * s;
  s = sdlInit(640, 480);
  sdlLoop(s);
  return 0;
}

/*!\brief Initialiser SDL pour OpenGL */
static SDL_Surface * sdlInit(int w, int h) {
  SDL_Surface * s;
  const SDL_VideoInfo * vi = NULL;
  if ( SDL_Init(SDL_INIT_VIDEO) < 0 ) {
    fprintf(stderr, "Impossible d'initialiser SDL: %s\n", SDL_GetError());
    exit(1);
  }
  if( (vi = SDL_GetVideoInfo()) == 0 ) {
    fprintf(stderr, "Echec de SDL_GetVideoInfo(): %s\n", SDL_GetError());
    exit(1);
  }
  SDL_GL_SetAttribute( SDL_GL_RED_SIZE,     8 );
  SDL_GL_SetAttribute( SDL_GL_GREEN_SIZE,   8 );
  SDL_GL_SetAttribute( SDL_GL_BLUE_SIZE,    8 );
  SDL_GL_SetAttribute( SDL_GL_DEPTH_SIZE,  16 );
  SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );
  if ( (s = SDL_SetVideoMode(w, h, vi->vfmt->BitsPerPixel,
			     SDL_OPENGL | SDL_RESIZABLE)) == NULL ) {
    fprintf(stderr, "Impossible d'ouvrir le mode video : %s\n", SDL_GetError());
    exit(1);
  }
  atexit(SDL_Quit);
  sdlInitGL(s);
  return s;
}

/*!\brief configurer les options GL avec la matrice de projection
 * réglée sur ortho.
 */
static void sdlInitGL(SDL_Surface * s) {
  glClearColor( 0, 0, 1, 1);
  glViewport(0, 0, s->w, s->h);
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  glOrtho(-2, 2, -2, 2, 0.1, 10);
  glMatrixMode( GL_MODELVIEW );
  glLoadIdentity( );
}

/*!\brief loop principale : tester les évènements puis dessiner
 * (boucle infinie).
 */
static void sdlLoop(SDL_Surface * s) {
  for(;;) {
    sdlManageEvents(s);
    sdlDraw(s);
  }
}

/*!\brief gestion des évènements, les évènements sont :
 * escape, q, Q ou clique sur le bouton fermer pour quitter
 */
static void sdlManageEvents(SDL_Surface * s) {
  SDL_Event event;
  while(SDL_PollEvent(&event)) 
    switch (event.type) {
    case SDL_KEYDOWN:
      switch(event.key.keysym.sym) {
      case SDLK_ESCAPE:
      case 'q':
	exit(0);
      default:
	break;
      }
      break;
    case SDL_QUIT:
      exit(0);
    default:
      break;
    }
}

/*\brief calcul une rotation dans le repère MODELVIEW puis
 * dessine le carre.
 */
static void sdlDraw(SDL_Surface * s) {
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  glRotatef(2, 0, 0, 1);
  glColor3f( 1, 1, 0);
  /* ***** Ancienne syntaxe *****
    glBegin(GL_QUADS);
    glVertex3f(-1.0f, 1.0f, -3.0f);
    glVertex3f(-1.0f,-1.0f, -3.0f);
    glVertex3f( 1.0f,-1.0f, -3.0f);
    glVertex3f( 1.0f, 1.0f, -3.0f);
    glEnd();
  */
  /* Depuis OpenGL v3.1+ et OpenGL-ES */
  GLfloat vertices[] = {
    -1.0f, 1.0f, -3.0f, 
    -1.0f,-1.0f, -3.0f, 
     1.0f,-1.0f, -3.0f, 
     1.0f, 1.0f, -3.0f }; 
  glEnableClientState(GL_VERTEX_ARRAY);
  glVertexPointer(3, GL_FLOAT, 0, vertices);
  glDrawArrays(GL_QUADS, 0, 4);
  glDisableClientState(GL_VERTEX_ARRAY);
  /* Fin de comparaison ancienne/nouvelle syntaxe */
  SDL_GL_SwapBuffers();
}
